# -*- coding: utf-8 -*-

# Copyright (C) 2004-2005 Johan Svedberg <johan@svedberg.com>

# This file is part of ontv.

# ontv is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# ontv is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with ontv; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os.path
from gettext import gettext as _

import gtk
import gtk.glade
import gconf

import Configuration
from GUI import GUI
from ontv import NAME, GLADE_DIR, LOCALE_DIR

gtk.glade.bindtextdomain(NAME.lower(), LOCALE_DIR)

class PreferencesDialog(GUI):
    def __init__(self, applet):
        GUI.__init__(self)
        self.applet = applet
        self.pw = applet.pw
        self.config = applet.config
        self.listings = applet.listings

        glade_file = os.path.join(GLADE_DIR, "ontv.glade")
        self.xml = gtk.glade.XML(glade_file, domain=NAME.lower())

        self.__get_widgets()
        self.__connect_widgets()
        self.dialog.set_icon(self.icon)
        self.__make_channels_treeview()
        self.__add_notifications()

        self.__selected_channels = self.config.selected_channels

    def __get_widgets(self):
        self.dialog = self.xml.get_widget("preferences_dialog")
        self.closebutton = self.xml.get_widget("closebutton")
        self.notebook = self.xml.get_widget("notebook")
        self.xmltv_file_entry = self.xml.get_widget("xmltv_file_entry")
        self.xmltv_browse_button = self.xml.get_widget("xmltv_browse_button")
        self.reload_xmltv_file_checkbutton = self.xml.get_widget("reload_xmltv_file_checkbutton")
        self.reload_xmltv_file_spinbutton = self.xml.get_widget("reload_xmltv_file_spinbutton")
        self.source_name_label = self.xml.get_widget("source_name_label")
        self.source_url_label = self.xml.get_widget("source_url_label")
        self.generator_name_label = self.xml.get_widget("generator_name_label")
        self.generator_url_label = self.xml.get_widget("generator_url_label")
        self.date_generated_label = self.xml.get_widget("date_generated_label")
        self.current_programs_checkbutton = self.xml.get_widget("current_programs_checkbutton")
        self.upcoming_programs_checkbutton = self.xml.get_widget("upcoming_programs_checkbutton")
        self.channels_treeview = self.xml.get_widget("channels_treeview")

    def __connect_widgets(self):
        self.closebutton.connect("clicked", self.__closebutton_clicked)
        self.xmltv_file_entry.connect("activate", self.__xmltv_file_entry_activate)
        self.xmltv_file_entry.connect("changed", self.__xmltv_file_entry_changed)
        self.xmltv_browse_button.connect("clicked", self.__xmltv_browse_button_clicked)
        self.reload_xmltv_file_checkbutton.connect("toggled", self.__reload_xmltv_file_checkbutton_toggled)
        self.reload_xmltv_file_spinbutton.connect("value_changed", self.__reload_xmltv_file_spinbutton_value_changed)
        self.current_programs_checkbutton.connect("toggled", self.__current_programs_checkbutton_toggled)
        self.upcoming_programs_checkbutton.connect("toggled", self.__upcoming_programs_checkbutton_toggled)

    def __make_channels_treeview(self):
        ls = gtk.ListStore(bool, gtk.gdk.Pixbuf, str)
        self.channels_treeview.set_model(ls)
        self.channels_treeview.set_enable_search(False)

        crtoggle = gtk.CellRendererToggle()
        crtoggle.set_property("xpad", 6)
        crtoggle.set_property("ypad", 3)
        crtoggle.set_property("activatable", True)
        crtoggle.connect("toggled", self.__channels_treeview_toggled, ls)
        selected_column = gtk.TreeViewColumn("Selected", crtoggle)
        selected_column.add_attribute(crtoggle, "active", 0)
        self.channels_treeview.append_column(selected_column)

        crpixbuf = gtk.CellRendererPixbuf()
        crpixbuf.set_property("xpad", 3)
        crpixbuf.set_property("ypad", 3)
        logo_column = gtk.TreeViewColumn("Logo", crpixbuf, pixbuf=1)
        self.channels_treeview.append_column(logo_column)

        crtext = gtk.CellRendererText()
        crtext.set_property("xpad", 3)
        crtext.set_property("ypad", 3)
        channel_column = gtk.TreeViewColumn("Channel", crtext, markup=2)
        self.channels_treeview.append_column(channel_column)

    def __add_notifications(self):
        self.config.add_notify(Configuration.KEY_XMLTV_FILE,
                               self.__xmltv_file_key_changed)
        self.config.add_notify(Configuration.KEY_AUTO_RELOAD_XMLTV_FILE,
                               self.__auto_reload_xmltv_file_key_changed)
        self.config.add_notify(Configuration.KEY_AUTO_RELOAD_XMLTV_FILE_RATE,
                               self.__auto_reload_xmltv_file_rate_key_changed)
        self.config.add_notify(Configuration.KEY_DISPLAY_CURRENT_PROGRAMS,
                               self.__display_current_programs_key_changed)
        self.config.add_notify(Configuration.KEY_DISPLAY_UPCOMING_PROGRAMS,
                               self.__display_upcoming_programs_key_changed)
        self.config.add_notify(Configuration.KEY_SELECTED_CHANNELS,
                               self.__selected_channels_key_changed)

    def update_channels_treeview(self):
        model = self.channels_treeview.get_model()
        model.clear()
        channels = []
        for channel in self.listings.values():
            channels.append(channel)
        channels.sort()
        markup = "<span weight=\"bold\">%s</span>"
        for channel in channels:
            if channel.display_name in self.__selected_channels:
                model.append([True, channel.logo, markup %
                             channel.display_name])
            else:
                model.append([False, channel.logo, markup %
                             channel.display_name])

    def update_xmltv_data(self):
        data = self.applet.xmltvfile.data
        data_map = {self.source_name_label: "source-info-name",
                    self.source_url_label: "source-info-url",
                    self.generator_name_label: "generator-info-name",
                    self.generator_url_label: "generator-info-url",
                    self.date_generated_label: "date"}

        for label in data_map.keys():
            if data.has_key(data_map[label]) and data[data_map[label]] != label.get_text():
                label.set_text(data[data_map[label]])
            else:
                label.set_text(_("N/A"))

    def show(self, uicomponent=None, verb=None):
        self.xmltv_file_entry.set_text(self.config.xmltv_file)
        self.reload_xmltv_file_checkbutton.set_active(self.config.auto_reload_xmltv_file)
        self.reload_xmltv_file_spinbutton.set_value(self.config.auto_reload_xmltv_file_rate)
        self.update_xmltv_data()
        self.current_programs_checkbutton.set_active(self.config.display_current_programs)
        self.upcoming_programs_checkbutton.set_active(self.config.display_upcoming_programs)

        self.update_channels_treeview()

        self.dialog.show()

    def __closebutton_clicked(self, button):
        self.dialog.hide()

    def __xmltv_file_entry_changed(self, entry):
        if len(entry.get_text()) > 0:
            self.reload_xmltv_file_checkbutton.set_sensitive(True)
            self.current_programs_checkbutton.set_sensitive(True)
            self.upcoming_programs_checkbutton.set_sensitive(True)
        else:
            self.reload_xmltv_file_checkbutton.set_sensitive(False)
            self.current_programs_checkbutton.set_sensitive(False)
            self.upcoming_programs_checkbutton.set_sensitive(False)

    def __xmltv_file_entry_activate(self, entry):
        self.config.xmltv_file = entry.get_text()

    def __xmltv_browse_button_clicked(self, button):
        fd = FilechooserDialog()
        response = fd.run()
        if response == gtk.RESPONSE_OK:
            self.config.xmltv_file = fd.get_filename()
            fd.hide()
        elif response == gtk.RESPONSE_CANCEL:
            fd.hide()
        fd.hide()

    def __reload_xmltv_file_checkbutton_toggled(self, checkbutton):
        self.config.auto_reload_xmltv_file = checkbutton.get_active()
        if checkbutton.get_active():
            self.reload_xmltv_file_spinbutton.set_sensitive(True)
        else:
            self.reload_xmltv_file_spinbutton.set_sensitive(False)

    def __reload_xmltv_file_spinbutton_value_changed(self, spinbutton):
        self.config.auto_reload_xmltv_file_rate = spinbutton.get_value_as_int()

    def __current_programs_checkbutton_toggled(self, checkbutton):
        self.config.display_current_programs = checkbutton.get_active()

    def __upcoming_programs_checkbutton_toggled(self, checkbutton):
        self.config.display_upcoming_programs = checkbutton.get_active()

    def __channels_treeview_toggled(self, cell, path, model):
        toggled = model[path][0]
        channel = model[path][2][20:-7]
        if not toggled:
            self.__selected_channels.append(channel)
        else:
            self.__selected_channels.remove(channel)
        self.config.selected_channels = self.__selected_channels

    def __xmltv_file_key_changed(self, client, cnxn_id, entry, data):
        if not entry.value:
            self.xmltv_file_entry.set_text("")
        elif entry.value.type == gconf.VALUE_STRING:
            self.xmltv_file_entry.set_text(entry.value.to_string())
            self.applet.xmltvfile.path = entry.value.to_string()
            self.applet.reload_xmltvfile_in_thread()
        else:
            self.xmltv_file_entry.set_text("!type error!")

    def __auto_reload_xmltv_file_key_changed(self, client, cnxn_id, entry, data):
        if not entry.value:
            self.reload_xmltv_file_checkbutton.set_active(True)
        elif entry.value.type == gconf.VALUE_BOOL:
            self.reload_xmltv_file_checkbutton.set_active(entry.value.get_bool())
            if entry.value.get_bool():
                self.applet.start_auto_reload(self.config.auto_reload_xmltv_file_rate)
            else:
                self.applet.stop_auto_reload()
        else:
            self.reload_xmltv_file_checkbutton.set_active(True)

    def __auto_reload_xmltv_file_rate_key_changed(self, client, cnxn_id, entry, data):
        if not entry.value:
            self.reload_xmltv_file_spinbutton.set_value(12)
        elif entry.value.type == gconf.VALUE_INT:
            value = entry.value.get_int()
            self.reload_xmltv_file_spinbutton.set_value(value)
            if self.config.auto_reload_xmltv_file:
                self.applet.stop_auto_reload()
                self.applet.start_auto_reload(value)
        else:
            self.reload_xmltv_file_spinbutton.set_value(12)

    def __display_current_programs_key_changed(self, client, cnxn_id, entry, data):
        if not entry.value:
            self.current_programs_checkbutton.set_active(True)
        elif entry.value.type == gconf.VALUE_BOOL:
            value = entry.value.get_bool()
            self.current_programs_checkbutton.set_active(value)
            if value:
                self.pw.cpt.show()
            else:
                self.pw.cpt.hide()
        else:
            self.current_programs_checkbutton.set_active(True)

    def __display_upcoming_programs_key_changed(self, client, cnxn_id, entry, data):
        if not entry.value:
            self.upcoming_programs_checkbutton.set_active(True)
        elif entry.value.type == gconf.VALUE_BOOL:
            value = entry.value.get_bool()
            self.upcoming_programs_checkbutton.set_active(value)
            if value:
                self.pw.upt.show()
            else:
                self.pw.upt.hide()
        else:
            self.upcoming_programs_checkbutton.set_active(True)

    def __selected_channels_key_changed(self, client, cnxn_id, entry, data):
        model = self.channels_treeview.get_model()
        if not entry.value:
            model.clear()
        elif entry.value.type == gconf.VALUE_LIST:
            channels = [value.to_string() for value in entry.value.get_list()]
            for row in model:
                channel = row[2][20:-7]
                if channel in channels:
                    row[0] = True
                else:
                    row[0] = False
            self.pw.update()

class FilechooserDialog(GUI):
    def __init__(self):
        GUI.__init__(self)
        self.dialog = gtk.FileChooserDialog(title=_("Select XMLTV file..."),
                                            action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                            buttons=(gtk.STOCK_CANCEL,
                                            gtk.RESPONSE_CANCEL,
                                            gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        self.dialog.set_icon(self.icon)
        xml_filter = gtk.FileFilter()
        xml_filter.set_name(_("XML files"))
        xml_filter.add_mime_type("text/xml")
        self.dialog.add_filter(xml_filter)
        all_filter = gtk.FileFilter()
        all_filter.set_name(_("All files"))
        all_filter.add_pattern("*")
        self.dialog.add_filter(all_filter)

    def hide(self):
        self.dialog.hide()

    def get_filename(self):
        return self.dialog.get_filename()

    def run(self):
        return self.dialog.run()
